package ch.epfl.ppo.collections;

import java.util.Iterator;
import java.util.NoSuchElementException;

public class SimpleHashSet<E> implements Set<E> {
    private int size;
    private ListSet<E>[] table;

    public SimpleHashSet() {
        @SuppressWarnings("unchecked")
        ListSet<E>[] table = new ListSet[10];
        for (int i = 0; i < table.length; ++i)
            table[i] = new ListSet<>();

        this.size = 0;
        this.table = table;
    }

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public int size() {
        return size;
    }

    @Override
    public void add(E newElem) {
        ListSet<E> list = listFor(newElem);
        size -= list.size();
        list.add(newElem);
        size += list.size();
    }

    @Override
    public void remove(E elem) {
        ListSet<E> list = listFor(elem);
        size -= list.size();
        list.remove(elem);
        size += list.size();
    }

    @Override
    public boolean contains(E elem) {
        return listFor(elem).contains(elem);
    }

    @Override
    public Iterator<E> iterator() {
        return new SHSIterator<>(table, size);
    }

    private ListSet<E> listFor(E elem) {
        return table[Math.abs(elem.hashCode() % table.length)];
    }

    private static final class SHSIterator<E> implements Iterator<E> {
        private final ListSet<E>[] table;
        private int remaining;
        private int i;
        private Iterator<E> it;

        public SHSIterator(ListSet<E>[] table, int size) {
            this.table = table;
            this.remaining = size;
            this.i = 0;
            this.it = table[i].iterator();
        }

        @Override
        public boolean hasNext() {
            return remaining > 0;
        }

        @Override
        public E next() {
            if (! hasNext())
                throw new NoSuchElementException();
            --remaining;
            while (! it.hasNext())
                it = table[++i].iterator();
            return it.next();
        }

        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
