package ch.epfl.ppo.collections;

import java.util.Iterator;
import java.util.NoSuchElementException;

public class SimpleHashSet3<E> implements Set<E> {
    private int size;
    private ListSet<E>[] table;

    public SimpleHashSet3() {
        @SuppressWarnings("unchecked")
        ListSet<E>[] table = new ListSet[10];
        for (int i = 0; i < table.length; ++i)
            table[i] = new ListSet<>();

        this.size = 0;
        this.table = table;
    }

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public int size() {
        return size;
    }

    @Override
    public void add(E newElem) {
        ListSet<E> list = listFor(newElem);
        size -= list.size();
        list.add(newElem);
        size += list.size();
    }

    @Override
    public void remove(E elem) {
        ListSet<E> list = listFor(elem);
        size -= list.size();
        list.remove(elem);
        size += list.size();
    }

    @Override
    public boolean contains(E elem) {
        return listFor(elem).contains(elem);
    }

    @Override
    public Iterator<E> iterator() {
        return new Iterator<E>() {
            private int remaining = size;
            private int i = 0;
            private Iterator<E> it = table[i].iterator();

            @Override
            public boolean hasNext() {
                return remaining > 0;
            }

            @Override
            public E next() {
                if (! hasNext())
                    throw new NoSuchElementException();
                --remaining;
                while (! it.hasNext())
                    it = table[++i].iterator();
                return it.next();
            }

            @Override
            public void remove() {
                it.remove();
                --size;
            }
        };
    }

    private ListSet<E> listFor(E elem) {
        return table[Math.abs(elem.hashCode() % table.length)];
    }
}
